__author__ = 'TNF'

'''
This code reads a user's symbol list text file and retrieves data using common start and end dates specified in the code.
Each symbol is then saved in its own csv file.

This is a general use case for downloading and saving data by symbol with no analytics performed on the data.
Common use cases are symbol lists derived from higher order processes (fundamental data) that are being turned
over to higher frequency data for entry/exit opportunities.

This code runs against Tick Data's Sandbox which is not a Production level server.  Download times may be slower
than the AWS load-balanced Production environment.

'''

import requests
import os.path
import urllib.request as url
import urllib.error as urlerror

# Location of symbol list to extract
symbollist_path = "D:\My_Documents\TickData\Product\TickAPI\Python_Examples\Download_Symbol_List_Sandbox\SymbolList.csv"

# Location to save output data
save_path = 'D:\My_Documents\TickData\Product\TickAPI\Python_Examples\Download_Symbol_List_Sandbox\_Output'


## API credentials and Parameters
api_user = "APIdemo"
api_pwd = "bACH93bH"
StartDate = '01/04/2016'
EndDate = '01/08/2016'


## Read Symbol List File
with open(symbollist_path) as f2:
    symbollist = f2.readlines()
symbollist = [x.strip() for x in symbollist]
f2.close()


## Request Data
'''

The url was built using Tick Data's RequestBuilder: https://sandbox-tickapi.tickdata.com/
Symbol, Start Data, and End Date have been parameterized.
One minute bar data has been specified in the url.

'''

for symbol in symbollist:
    print("Downloading: ",symbol)

    ## Build the url
    urlp = 'https://sandbox-tickapi.tickdata.com//stream?COLLECTION_TYPE=COLLECTION_TYPE.US_TED' \
           '&EXTRACT_TYPE=COLLECTOR_SUBTYPE_US_TICK_TIMEBAR' \
           '&START_DATE=' + StartDate + \
           '&END_DATE=' + EndDate + \
           '&OUTPUT_NAMING_CONVENTION=0' \
           '&REQUESTED_DATA=' + symbol + \
           '&SESSIONS=MARKET' \
           '&GRANULARITY=1' \
           '&COMPRESSION_TYPE=NONE' \
           '&SELECTED_FIELDS=DATE_FIELD|TICK_TIME_FIELD|CLOSE_PRICE_FIELD|VOLUME_FIELD'


    # create an OpenerDirector with support for Basic HTTP Authentication
    auth_handler = url.HTTPBasicAuthHandler()

    # adding authentication details the handler
    auth_handler.add_password(realm='TW Server',
                              uri=urlp,
                              user=api_user,
                              passwd=api_pwd)

    # building an opener auth handler
    opener = url.build_opener(auth_handler)
    url.install_opener(opener)

    # try block
    try:
        # opening the url
        f = url.urlopen(urlp)
        # opening the file for each symbol in the write binary mode
        with open(symbol + ".csv", "wb") as file:
            # writing the data from the URL to the file
            file.write(f.read())
    # exception handling
    except urlerror.HTTPError as e:
        # checking the HTTPError code
        if hasattr(e, 'code'):
            # checking for any other error apart from URL error
            if e.code != 401:
                # printing the error
                print('I have an error')
                print(e.code)
        else:
            # printing the urlerror
            print(e.headers)

